Game = {}

local PrintPlayerDeadMessage = false
local RespawnTimer = 3

function Game.Setup()
	grid = {
		cellSize 		= 32,
		moveCellSize	= 16
	}

	KeyEvent = {
		keyPressed 		= false, 
		key 			= "z"
	}

	EntityManager.Init()

	TILEMAP_WIDTH = 31
	TILEMAP_HEIGHT = 22
	TILEMAP_X = 0
	TILEMAP_Y = 0

	currentLevel = 1
	
	tilemap = {}

	floorTileSpritebatch = love.graphics.newSpriteBatch(floorTileImage,682)
end

function Game.Update(dt)
	EntityManager.Update(dt)
	
	bump.collide()
	KeyEvent.keyPressed = false

	print(love.timer.getFPS())

	GridFlicker(dt)

	if PrintPlayerDeadMessage then 
		if RespawnTimer > 0 then 
			RespawnTimer = RespawnTimer - dt
		else
			PrintPlayerDeadMessage = false
			RespawnTimer = 3
			Game.PlayerRespawn()
		end
	end
end

function Game.Draw()
	--setup locals for optimization
	local gfx = love.graphics
	local grd = grid

	if screen.gridDraw then
		gfx.setColor(255,0,0,75)
	else
		gfx.setColor(150,0,0,75)
	end

	love.graphics.draw(floorTileSpritebatch,TILEMAP_X,TILEMAP_Y)

	--OLD Tilemap Draw Stuff
	--I'm keeping it here, because I prefer its look to the current method...
	--...but line drawing with the AA on in LOVE is REALLY slow.
	--Keeping it here in case this is improved in later versions of LOVE...
	--[[for tx=1,TILEMAP_WIDTH do
		for ty=1,TILEMAP_HEIGHT do
			local dX = tx * grd.cellSize - grd.cellSize + TILEMAP_X
			local dY = ty * grd.cellSize - grd.cellSize + TILEMAP_Y

			if tilemap[ty][tx] ~= "X" then
				gfx.rectangle("line",dX,dY,grd.cellSize,grd.cellSize)
			end
		end
	end --]]

	gfx.setColor(255,255,255,255)  

	EntityManager.Draw()

	if PrintPlayerDeadMessage then
		
		love.graphics.setColor(255,0,0,200)
		love.graphics.rectangle("fill",432,336,128,48)
		love.graphics.setColor(0,0,0,255)
		love.graphics.print("YOU DIED!",464,352)
		gfx.setColor(255,255,255,255)  
	end

 	local blendmode = gfx.getBlendMode()
	gfx.setBlendMode("multiplicative")
	gfx.setColor(255, 255, 255,125)
	gfx.draw(screenImage,0,0)
	gfx.setBlendMode(blendmode)
end

function Game.PlayerDied()
	PrintPlayerDeadMessage = true
end

function Game.PlayerRespawn()
	LevelLoader.ChangeLevel(currentLevel)
end

function GridFlicker(dt)
	--setup locals for optimization
	local scrn = screen
	local r = math.random

	if scrn.flickerTimer > 0 then 
		scrn.flickerTimer = scrn.flickerTimer - dt
	else
		scrn.gridDraw = false

		if scrn.flickerInterval > 0 then
			scrn.flickerInterval = scrn.flickerInterval - dt
		else
			scrn.gridDraw = true
			scrn.flickerTimer = 1 * r(3,25)
			scrn.flickerInterval = 0.075 * r(1,2.5)
		end
	end
end