Player = class('Player',Entity) 
local super = Entity

--setup locals for perf
local gfx = love.graphics

local PLAYER_WIDTH = 16
local PLAYER_HEIGHT = 32
local PLAYER_WALK_SPEED = 250
local PLAYER_RUN_SPEED = 400
local PLAYER_MOVE_DELAY = 0.075

function Player:initialize(x,y,name)
	super.initialize(self,x,y)
	self.name = name
	self.moving = false
	self.moveSpeed = PLAYER_WALK_SPEED
	self.desX = self.x
	self.desY = self.y
	self.lastX = self.x
	self.lastY = self.y
	self.dir = {x = 1, y = 0}
	self:SetupBBox("player", self.x, self.y, PLAYER_WIDTH, PLAYER_HEIGHT)
	self.moveDelayTimer = PLAYER_MOVE_DELAY
end

function Player:Update(dt)
	if love.keyboard.isDown("lshift") then 
		self.moveSpeed = PLAYER_RUN_SPEED
	else	
		self.moveSpeed = PLAYER_WALK_SPEED
	end

	if love.keyboard.isDown("right") then
		self.dir = {x = 1, y = 0}
		if self.moveDelayTimer > 0 then
			self.moveDelayTimer = self.moveDelayTimer - dt
		else
			self:Move(1,0)
		end
	elseif love.keyboard.isDown("left") then
		self.dir = {x = -1, y = 0}
		if self.moveDelayTimer > 0 then
			self.moveDelayTimer = self.moveDelayTimer - dt
		else
			self:Move(-1,0)
		end
	elseif love.keyboard.isDown("down") then
		self.dir = {x = 0, y = 1}
		if self.moveDelayTimer > 0 then
			self.moveDelayTimer = self.moveDelayTimer - dt
		else
			self:Move(0,1)
		end
	elseif love.keyboard.isDown("up") then
		self.dir = {x = 0, y = -1}
		if self.moveDelayTimer > 0 then
			self.moveDelayTimer = self.moveDelayTimer - dt
		else
			self:Move(0,-1)
		end
	else
		self.moveDelayTimer = PLAYER_MOVE_DELAY
	end

	if KeyEvent.keyPressed then 
		if KeyEvent.key == "z" then
			PlayerSword:new(self,self.x + 26 * self.dir.x - ((32 - PLAYER_WIDTH)/2),self.y + 32 * self.dir.y)
			print("PRESSED")
		elseif KeyEvent.key == "x" then
			PlayerBullet:new(self,self.x + 26 * self.dir.x - ((32 - PLAYER_WIDTH)/2),self.y + 32 * self.dir.y)
			print("PRESSED")
		end
	end

	self:UpdateMovement(dt)
	self:UpdateBBox()
end

function Player:Draw()
	gfx.setColor(50,255,255,255)
	gfx.rectangle("line",self.x,self.y,PLAYER_WIDTH,PLAYER_HEIGHT)
	if self.dir.y ~= -1 then gfx.rectangle("line",self.x+(PLAYER_WIDTH/4)+ (PLAYER_WIDTH/2 * self.dir.x),self.y+(PLAYER_HEIGHT/4) + (-PLAYER_HEIGHT/8),PLAYER_WIDTH/2,PLAYER_HEIGHT/2) end
	gfx.setColor(255,255,255)
	--love.graphics.drawq(img2, diffuse2, self.x, self.y)
end

function Player:Kill()
	Pop:new(self.x+6,self.y+12,{50,255,255})
	bump.remove(self.bbox)
	super.Kill(self)
end


function Player:Collision(collidedWith, dx, dy)
	if collidedWith.name == "wall" then
	  	self.x = math.floor(self.lastX / grid.moveCellSize) * grid.moveCellSize
	  	self.y = math.floor(self.lastY / grid.moveCellSize) * grid.moveCellSize
	  	self.desX = self.x
	  	self.desY = self.y
	  	self.moving = false
	elseif collidedWith.name == "exit" then 
		local lvl = currentLevel + 1
		LevelLoader.ChangeLevel(lvl)
	elseif collidedWith.name == "monster" then
		self:Eliminate()
	end
end

function Player:Eliminate()
	Game.PlayerDied()
	self:Kill()
end

function Player:Move(mX,mY)
	local mX = mX or 0
	local mY = mY or 0

	self.desX = math.floor((self.x + grid.moveCellSize * mX)/grid.moveCellSize) * grid.moveCellSize
	self.desY = math.floor((self.y + grid.moveCellSize * mY)/grid.moveCellSize) * grid.moveCellSize

	if self.moving == false then
		

		self.moving = true
	end
end

function Player:UpdateMovement(dt)
	self.lastX = self.x
	self.lastY = self.y

	if self.moving then 
		if self.x ~= self.desX then 
			if self.desX > self.x then
				if ((self.desX - self.x) < self.moveSpeed * dt) then
					self.x = self.desX
				else
					self.x = self.x + self.moveSpeed * dt
				end
			else
				if (math.abs(self.x - self.desX) < self.moveSpeed * dt) then
					self.x = self.desX
				else
					self.x = self.x - self.moveSpeed * dt
				end
			end
		elseif self.y ~= self.desY then 
			if self.desY > self.y then
				if ((self.desY - self.y) < self.moveSpeed * dt) then
					self.y = self.desY
				else
					self.y = self.y + self.moveSpeed * dt
				end
			else
				if (math.abs(self.y - self.desY) < self.moveSpeed * dt) then
					self.y = self.desY
				else
					self.y = self.y - self.moveSpeed * dt
				end
			end
		else
			self.moving = false
		end
	end
end