local gui = require "quickie"

function love.load()
	screen = {
		width = 1280,
		height = 720,
		fullscreen = false,
	}

	level = {
		name = "1"
	}
	input = {text = level.name}

	grid = {
		cellSize = 32
	}


	mouse = {
		x = love.mouse.getX(),
		y = love.mouse.getY(),
		tX = math.floor(love.mouse.getX()/grid.cellSize) + 1,
		tY = math.floor(love.mouse.getY()/grid.cellSize) + 1
	}

	love.graphics.setMode(screen.width,screen.height,screen.fullscreen)


	TilemapInit()

	things = {}
	currentThing = "wall"

	thingPicker = {
		x = 1008,
		y = 208,
		w = 3,
		h = 3,
		tS = 32,
		pal = {
			{"wall","player","walker"},
			{"exit","",""},
			{"","",""}
		}
	}

	currentBrushSize = 1

	brushSizePicker = {
		x = 1008,
		y = 352,
		w = 3,
		h = 2,
		tS = 32,
		pal = {
			{1,2,3},
			{4,8,16}
		}
	}
end

function love.update(dt)
	mouse = {
		x = love.mouse.getX(),
		y = love.mouse.getY(),
		tX = math.floor(love.mouse.getX()/grid.cellSize) + 1,
		tY = math.floor(love.mouse.getY()/grid.cellSize) + 1
	}
	
	local rMTX = mouse.tX
	local rMTY = mouse.tY
	if mouse.tX > tilemapWidth then mouse.tX = tilemapWidth end
	if mouse.tY > tilemapHeight then mouse.tY = tilemapHeight end

	if love.mouse.isDown("l") then 
		if ((rMTX <= tilemapWidth) and (rMTY <= tilemapHeight)) then 
			if currentThing == "wall" then
				PlaceThing("X",mouse.tX,mouse.tY,currentBrushSize)
			else
				PlaceThing(currentThing,mouse.tX,mouse.tY,currentBrushSize)
			end
		end
	end

	if love.mouse.isDown("r") then 
		if ((rMTX <= tilemapWidth) and (rMTY <= tilemapHeight)) then 
			PlaceThing("",mouse.tX,mouse.tY,currentBrushSize)
		end
	end

	if gui.Input ({info = input, pos = {1085, 535}, size = {60, 16}}) then 
		print('Level changed:', input.text)
		level.name = input.text 
	end
	 if gui.Button ({text = "Save Level", pos = {1032, 559}, size = {96, 16}}) then Save() end
	 if gui.Button ({text = "Load Level", pos = {1032, 583}, size = {96, 16}}) then Load() end
	
end

function love.draw()
	for tx=1,tilemapWidth do
		for ty=1,tilemapHeight do
			local dX = tx * grid.cellSize - grid.cellSize + tilemapX
			local dY = ty * grid.cellSize - grid.cellSize + tilemapY

			if tilemap[ty][tx] ~= "X" then
				love.graphics.setColor(255,0,0,100)
				love.graphics.rectangle("line",dX,dY,grid.cellSize,grid.cellSize)
				love.graphics.setColor(255,255,255,255)
			end
			
			if tilemap[ty][tx] == "player" then
				love.graphics.setColor(50,255,255,255)
				love.graphics.rectangle("line",dX,dY,16,32)
				love.graphics.setColor(255,255,255,255)
			elseif tilemap[ty][tx] == "walker" then
				love.graphics.setColor(0,255,0,150)
				love.graphics.rectangle("line",dX,dY,16,32)
				love.graphics.setColor(255,255,255,255)
			elseif tilemap[ty][tx] == "exit" then 
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("fill",dX,dY,4,4)
				love.graphics.rectangle("fill",dX+8,dY,4,4)
				love.graphics.rectangle("fill",dX+16,dY,4,4)
				love.graphics.rectangle("fill",dX+24,dY,4,4)
				love.graphics.rectangle("fill",dX+4,dY+4,4,4)
				love.graphics.rectangle("fill",dX+12,dY+4,4,4)
				love.graphics.rectangle("fill",dX+20,dY+4,4,4)
				love.graphics.rectangle("fill",dX+28,dY+4,4,4)

				love.graphics.rectangle("fill",dX,dY+8,4,4)
				love.graphics.rectangle("fill",dX+8,dY+8,4,4)
				love.graphics.rectangle("fill",dX+16,dY+8,4,4)
				love.graphics.rectangle("fill",dX+24,dY+8,4,4)
				love.graphics.rectangle("fill",dX+4,dY+12,4,4)
				love.graphics.rectangle("fill",dX+12,dY+12,4,4)
				love.graphics.rectangle("fill",dX+20,dY+12,4,4)
				love.graphics.rectangle("fill",dX+28,dY+12,4,4)

				love.graphics.rectangle("fill",dX,dY+16,4,4)
				love.graphics.rectangle("fill",dX+8,dY+16,4,4)
				love.graphics.rectangle("fill",dX+16,dY+16,4,4)
				love.graphics.rectangle("fill",dX+24,dY+16,4,4)
				love.graphics.rectangle("fill",dX+4,dY+20,4,4)
				love.graphics.rectangle("fill",dX+12,dY+20,4,4)
				love.graphics.rectangle("fill",dX+20,dY+20,4,4)
				love.graphics.rectangle("fill",dX+28,dY+20,4,4)

				love.graphics.rectangle("fill",dX,dY+24,4,4)
				love.graphics.rectangle("fill",dX+8,dY+24,4,4)
				love.graphics.rectangle("fill",dX+16,dY+24,4,4)
				love.graphics.rectangle("fill",dX+24,dY+24,4,4)
				love.graphics.rectangle("fill",dX+4,dY+28,4,4)
				love.graphics.rectangle("fill",dX+12,dY+28,4,4)
				love.graphics.rectangle("fill",dX+20,dY+28,4,4)
				love.graphics.rectangle("fill",dX+28,dY+28,4,4)
			end
		end
	end

	love.graphics.print("DUNGEONMAN ED",1008,32)
	
	DrawInfoBox(1008,64)
	DrawThingPicker()
	DrawBrushSizePicker()
	DrawLevelDetails(1008,496)
	gui.core.draw()
end

function love.mousepressed(b)
		ThingPickerClick()
		BrushSizePickerClick()
end

function love.keypressed(k,code)
	if k == "s" then Save()
	elseif k == "l" then Load() end
	 gui.keyboard.pressed(k, code)
end

function Save()
	success = love.filesystem.write( level.name..".dmlvl",table.show(tilemap,"loadedTilemap"))
	if success == false then print("uh-oh!") end
end

function Load()
	local fname = level.name..".dmlvl" 
	if love.filesystem.exists(fname) then
		chunk = love.filesystem.load( level.name..".dmlvl" )
		chunk()
		tilemap = loadedTilemap
	else 
		print("Level "..fname.." doesn't exist! Creating blank level...")
		TilemapInit()
	end
end

function DrawLevelDetails(x,y)
	local dX = x or 0
	local dY = y or 0
	local indent = 24

	love.graphics.print("LEVEL DETAILS START",dX,dY)
	love.graphics.print("LEVEL: "..level.name..".dmlvl",dX+indent,dY+16)
	love.graphics.print("TARGET:",dX+indent,dY+40)
	love.graphics.print(".dmlvl",dX+indent+114,dY+40)
	--love.graphics.print("PRESS 'S' TO SAVE LEVEL",dX+indent,dY+48)
	--love.graphics.print("PRESS 'L' TO LOAD LEVEL",dX+indent,dY+64)
	love.graphics.print("LEVEL DETAILS END",dX,dY+111)
end

function DrawInfoBox(x,y)
	local dX = x or 0
	local dY = y or 0
	local indent = 24

	love.graphics.print("INFO START",dX,dY)
	love.graphics.print("Mouse Real X: "..mouse.x,dX+indent,dY+16)
	love.graphics.print("Mouse Real Y: "..mouse.y,dX+indent,dY+32)
	love.graphics.print("Mouse Tile X: "..mouse.tX,dX+indent,dY+48)
	love.graphics.print("Mouse Tile Y: "..mouse.tY,dX+indent,dY+64)
	love.graphics.print("INFO END",dX,dY+80)
end

function DrawThingPicker()
	love.graphics.print("THING PICKER",thingPicker.x,thingPicker.y)

	love.graphics.setColor(255,255,255,255)
	for tx=1,thingPicker.w do
		for ty=1,thingPicker.h do
			local dX = tx * thingPicker.tS - thingPicker.tS + thingPicker.x
			local dY = ty * thingPicker.tS - thingPicker.tS + thingPicker.y+18

			love.graphics.rectangle("line",dX,dY,thingPicker.tS,thingPicker.tS)

			if thingPicker.pal[ty][tx] == "wall" then
				--
				--WALL
				--
				if currentThing == "wall" then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
					love.graphics.setColor(255,255,255,255)
				end
				love.graphics.print("WALL",dX,dY+8)
			elseif thingPicker.pal[ty][tx] == "player" then
				--
				--PLAYER
				--
				if currentThing == "player" then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				
				love.graphics.setColor(50,255,255,255)
				love.graphics.rectangle("line",dX+10,dY+4,12,24)
				love.graphics.setColor(255,255,255,255)
			elseif thingPicker.pal[ty][tx] == "walker" then
				--
				--WALKER
				--
				if currentThing == "walker" then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				
				love.graphics.setColor(150,255,150,255)
				love.graphics.rectangle("line",dX+10,dY+4,12,24)
				love.graphics.setColor(255,255,255,255)
			elseif thingPicker.pal[ty][tx] == "exit" then
				--
				--WALKER
				--
				if currentThing == "exit" then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("fill",dX,dY,4,4)
				love.graphics.rectangle("fill",dX+8,dY,4,4)
				love.graphics.rectangle("fill",dX+16,dY,4,4)
				love.graphics.rectangle("fill",dX+24,dY,4,4)
				love.graphics.rectangle("fill",dX+4,dY+4,4,4)
				love.graphics.rectangle("fill",dX+12,dY+4,4,4)
				love.graphics.rectangle("fill",dX+20,dY+4,4,4)
				love.graphics.rectangle("fill",dX+28,dY+4,4,4)

				love.graphics.rectangle("fill",dX,dY+8,4,4)
				love.graphics.rectangle("fill",dX+8,dY+8,4,4)
				love.graphics.rectangle("fill",dX+16,dY+8,4,4)
				love.graphics.rectangle("fill",dX+24,dY+8,4,4)
				love.graphics.rectangle("fill",dX+4,dY+12,4,4)
				love.graphics.rectangle("fill",dX+12,dY+12,4,4)
				love.graphics.rectangle("fill",dX+20,dY+12,4,4)
				love.graphics.rectangle("fill",dX+28,dY+12,4,4)

				love.graphics.rectangle("fill",dX,dY+16,4,4)
				love.graphics.rectangle("fill",dX+8,dY+16,4,4)
				love.graphics.rectangle("fill",dX+16,dY+16,4,4)
				love.graphics.rectangle("fill",dX+24,dY+16,4,4)
				love.graphics.rectangle("fill",dX+4,dY+20,4,4)
				love.graphics.rectangle("fill",dX+12,dY+20,4,4)
				love.graphics.rectangle("fill",dX+20,dY+20,4,4)
				love.graphics.rectangle("fill",dX+28,dY+20,4,4)

				love.graphics.rectangle("fill",dX,dY+24,4,4)
				love.graphics.rectangle("fill",dX+8,dY+24,4,4)
				love.graphics.rectangle("fill",dX+16,dY+24,4,4)
				love.graphics.rectangle("fill",dX+24,dY+24,4,4)
				love.graphics.rectangle("fill",dX+4,dY+28,4,4)
				love.graphics.rectangle("fill",dX+12,dY+28,4,4)
				love.graphics.rectangle("fill",dX+20,dY+28,4,4)
				love.graphics.rectangle("fill",dX+28,dY+28,4,4)
			end
		end
	end

	love.graphics.print("Current: "..currentThing,thingPicker.x,thingPicker.y+18+(thingPicker.tS*thingPicker.h)+4)
end

function ThingPickerClick()
	local tPL = thingPicker.x
	local tPT = thingPicker.y + 18
	local tTS = thingPicker.tS
	local tPR = tPL + (thingPicker.w * tTS)
	local tPB = tPT + (thingPicker.h * tTS)

	if PointInRectangle(mouse.x,mouse.y,tPL,tPT,tPR,tPB) then
		for tx=1,thingPicker.w do
			for ty=1,thingPicker.h do
				local cX = tx * tTS - tTS + tPL
				local cY = ty * tTS - tTS + tPT

				if PointInRectangle(mouse.x,mouse.y,cX,cY,cX+tTS,cY+tTS) then
					currentThing = thingPicker.pal[ty][tx]
				end	
			end
		end
	end
end

function DrawBrushSizePicker()
	love.graphics.print("BRUSH SIZE",brushSizePicker.x,brushSizePicker.y)

	love.graphics.setColor(255,255,255,255)
	for tx=1,brushSizePicker.w do
		for ty=1,brushSizePicker.h do
			local dX = tx * brushSizePicker.tS - brushSizePicker.tS + brushSizePicker.x
			local dY = ty * brushSizePicker.tS - brushSizePicker.tS + brushSizePicker.y+18

			love.graphics.rectangle("line",dX,dY,brushSizePicker.tS,brushSizePicker.tS)

			if brushSizePicker.pal[ty][tx] == 1 then

				if currentBrushSize == 1 then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("fill",dX+15,dY+15,2,2)
			elseif brushSizePicker.pal[ty][tx] == 2 then

				if currentBrushSize == 2 then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("fill",dX+14,dY+14,4,4)
			elseif brushSizePicker.pal[ty][tx] == 3 then

				if currentBrushSize == 3 then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("fill",dX+13,dY+13,6,6)
			elseif brushSizePicker.pal[ty][tx] == 4 then

				if currentBrushSize == 4 then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("fill",dX+12,dY+12,8,8)
			elseif brushSizePicker.pal[ty][tx] == 8 then

				if currentBrushSize == 8 then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("fill",dX+8,dY+8,16,16)
			elseif brushSizePicker.pal[ty][tx] == 16 then

				if currentBrushSize == 16 then 
					love.graphics.setColor(0,255,0,150)
					love.graphics.rectangle("fill",dX,dY,thingPicker.tS,thingPicker.tS)
				end
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("fill",dX+2,dY+2,28,28)
			end

		end
	end

	love.graphics.print("Current: "..currentBrushSize.."x"..currentBrushSize,brushSizePicker.x,brushSizePicker.y+18+(brushSizePicker.tS*brushSizePicker.h)+4)
end

function BrushSizePickerClick()
	local bPL = brushSizePicker.x
	local bPT = brushSizePicker.y + 18
	local bTS = brushSizePicker.tS
	local bPR = bPL + (brushSizePicker.w * bTS)
	local bPB = bPT + (brushSizePicker.h * bTS)

	if PointInRectangle(mouse.x,mouse.y,bPL,bPT,bPR,bPB) then
		for tx=1,brushSizePicker.w do
			for ty=1,brushSizePicker.h do
				local cX = tx * bTS - bTS + bPL
				local cY = ty * bTS - bTS + bPT

				if PointInRectangle(mouse.x,mouse.y,cX,cY,cX+bTS,cY+bTS) then
					currentBrushSize = brushSizePicker.pal[ty][tx]
				end	
			end
		end
	end
end


function PointInRectangle(ax,ay,bx,by,br,bb)
	local isIt = false 

	if ((ax >= bx) and
		(ay >= by) and 
		(ax <= br) and 
		(ay <= bb)) 
	then
		isIt = true
	end

	return isIt
end

function PlaceThing(thing,tX,tY,brushSize)
	local thing = thing or "X"
	local tX = tX or 1
	local tY = tY or 1
	local brushSize = brushSize or 1

	if brushSize == 1 then
		tilemap[tY][tX] = thing
	else
		local w = brushSize+tX-1
		if w > tilemapWidth then w = tilemapWidth end
		local h = brushSize+tY-1
		if h > tilemapHeight then h = tilemapHeight end

		for x=tX,w do
			for y=tY,h do
					tilemap[y][x] = thing
			end
		end
	end
end

function table.show(t, name, indent)
   local cart     -- a container
   local autoref  -- for self references

   --[[ counts the number of elements in a table
   local function tablecount(t)
      local n = 0
      for _, _ in pairs(t) do n = n+1 end
      return n
   end
   ]]
   -- (RiciLake) returns true if the table is empty
   local function isemptytable(t) return next(t) == nil end

   local function basicSerialize (o)
      local so = tostring(o)
      if type(o) == "function" then
         local info = debug.getinfo(o, "S")
         -- info.name is nil because o is not a calling level
         if info.what == "C" then
            return string.format("%q", so .. ", C function")
         else 
            -- the information is defined through lines
            return string.format("%q", so .. ", defined in (" ..
                info.linedefined .. "-" .. info.lastlinedefined ..
                ")" .. info.source)
         end
      elseif type(o) == "number" or type(o) == "boolean" then
         return so
      else
         return string.format("%q", so)
      end
   end

   local function addtocart (value, name, indent, saved, field)
      indent = indent or ""
      saved = saved or {}
      field = field or name

      cart = cart .. indent .. field

      if type(value) ~= "table" then
         cart = cart .. " = " .. basicSerialize(value) .. ";\n"
      else
         if saved[value] then
            cart = cart .. " = {}; -- " .. saved[value] 
                        .. " (self reference)\n"
            autoref = autoref ..  name .. " = " .. saved[value] .. ";\n"
         else
            saved[value] = name
            --if tablecount(value) == 0 then
            if isemptytable(value) then
               cart = cart .. " = {};\n"
            else
               cart = cart .. " = {\n"
               for k, v in pairs(value) do
                  k = basicSerialize(k)
                  local fname = string.format("%s[%s]", name, k)
                  field = string.format("[%s]", k)
                  -- three spaces between levels
                  addtocart(v, fname, indent .. "   ", saved, field)
               end
               cart = cart .. indent .. "};\n"
            end
         end
      end
   end

   name = name or "__unnamed__"
   if type(t) ~= "table" then
      return name .. " = " .. basicSerialize(t)
   end
   cart, autoref = "", ""
   addtocart(t, name, indent)
   return cart .. autoref
end

function TilemapInit()
	tilemapWidth = 31
	tilemapHeight = 22
	tilemapX = 0
	tilemapY = 0

	tilemap = {
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""},
		{"","","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""}
	}
end